<?php
declare(strict_types=1);

namespace CakeProducts\Controller;

use Cake\Core\Configure;
use Cake\Log\Log;
use Cake\ORM\Table;
use Cake\ORM\TableRegistry;
use Cake\Utility\Text;
use CheeseCake\Controller\Traits\OverrideTableTrait;
;

/**
 * ProductCategories Controller
 *
 * @property \CakeProducts\Model\Table\ProductCategoriesTable $ProductCategories
 */
class ProductCategoriesController extends AppController
{
    use OverrideTableTrait;

    /**
     * @return void
     */
    public function initialize(): void
    {
        parent::initialize(); // TODO: Change the autogenerated stub
//        $this->_defaultTable = 'CakeProducts.ProductCategories';
//        $this->_tableConfigKey = 'CakeProducts.ProductCategories.table';
    }

    /**
     * Index method
     *
     * @return \Cake\Http\Response|null|void Renders view
     */
    public function index()
    {
        $query = $this->getTable()->find()
            ->contain(['ProductCatalogs', 'ParentProductCategories']);
        $productCategories = $this->paginate($query);

        $this->set(compact('productCategories'));
    }

    /**
     * View method
     *
     * @param string|null $id Product Category id.
     * @return \Cake\Http\Response|null|void Renders view
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
        $productCategory = $this->getTable()->get($id, contain: ['ProductCatalogs', 'ParentProductCategories', 'ChildProductCategories']);
        $this->set(compact('productCategory'));
    }

    /**
     * Add method
     *
     * @return \Cake\Http\Response|null|void Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $productCategoriesTable = $this->getTable();
        $productCategory = $productCategoriesTable->newEmptyEntity();
        if ($this->request->is('post')) {
            $postData = $this->request->getData();
            $saveOptions = [
                'associated' => [],
            ];
            if ($this->request->getSession()->read('Auth.User.id')) {
                $postData['created_by'] = $this->request->getSession()->read('Auth.User.id');
            }
            if (!array_key_exists('internal_id', $postData) || !$postData['internal_id']) {
                $postData['internal_id'] = Text::uuid();
            }
            $productCategory = $productCategoriesTable->patchEntity($productCategory, $postData, $saveOptions);
            if ($productCategory->getErrors()) {
                Log::debug(print_r('$productCategory->getErrors() next - failed to save from create new product category', true));
                Log::debug(print_r($productCategory->getErrors(), true));
            }
            if ($this->ProductCategories->save($productCategory, $saveOptions)) {
                $this->Flash->success(__('The product category has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The product category could not be saved. Please, try again.'));
        }
        $productCatalogs = $productCategoriesTable->ProductCatalogs->find('list', limit: 200)->all();
        $parentProductCategories = $productCategoriesTable->ParentProductCategories->find('treeList', limit: 200)->toArray();
        $this->set(compact('productCategory', 'productCatalogs', 'parentProductCategories'));
    }

    /**
     * Edit method
     *
     * @param string|null $id Product Category id.
     * @return \Cake\Http\Response|null|void Redirects on successful edit, renders view otherwise.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $productCategoriesTable = $this->getTable();
        $productCategory = $productCategoriesTable->get($id, contain: []);
        if ($this->request->is(['patch', 'post', 'put'])) {
            $postData = $this->request->getData();
            $productCategory = $productCategoriesTable->patchEntity($productCategory, $postData);
            if ($productCategoriesTable->save($productCategory)) {
                $this->Flash->success(__('The product category has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The product category could not be saved. Please, try again.'));
        }
        $productCatalogs = $productCategoriesTable->ProductCatalogs->find('list', limit: 200)->all();
        $parentProductCategories = $productCategoriesTable->ParentProductCategories->find('list', limit: 200)->all();
        $this->set(compact('productCategory', 'productCatalogs', 'parentProductCategories'));
    }

    /**
     * Delete method
     *
     * @param string|null $id Product Category id.
     * @return \Cake\Http\Response|null Redirects to index.
     * @throws \Cake\Datasource\Exception\RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $productCategoriesTable = $this->getTable();

        $productCategory = $productCategoriesTable->get($id);
        if ($productCategoriesTable->delete($productCategory)) {
            $this->Flash->success(__('The product category has been deleted.'));
        } else {
            $this->Flash->error(__('The product category could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }

    /**
     * @return \Cake\Http\Response|null|void Renders view
     */
    public function select()
    {
        $productCategoriesTable = $this->getTable();
        $productCategoriesTable->behaviors()->get('Tree')->setConfig([
            'scope' => [
                'product_catalog_id' => $this->request->getQuery('product_catalog_id', -1),
            ],
        ]);
        $productCategories = $productCategoriesTable
            ->find('treeList')
//            ->where(['product_catalog_id' => $this->request->getQuery('product_catalog_id', -1)])
            ->orderBy(['ProductCategories.name'])
            ->toArray();

        $this->set(compact('productCategories'));
    }
}
