<?php
declare(strict_types=1);

namespace CakeProducts\Controller;

use Cake\Core\Configure;
use Cake\Datasource\Exception\RecordNotFoundException;
use Cake\Http\Response;
use Cake\Log\Log;
use Cake\ORM\Table;
use Cake\ORM\TableRegistry;
use CakeProducts\Controller\AppController;
use CheeseCake\Controller\Traits\OverrideTableTrait;
;
use CakeProducts\Model\Enum\ProductCategoryAttributeTypeId;
use CakeProducts\Model\Table\ProductCategoryAttributesTable;

/**
 * ProductCategoryAttributes Controller
 *
 * @property ProductCategoryAttributesTable $ProductCategoryAttributes
 */
class ProductCategoryAttributesController extends AppController
{
    use OverrideTableTrait;

    /**
     * @return void
     */
    public function initialize(): void
    {
        parent::initialize(); // TODO: Change the autogenerated stub
//        $this->_defaultTable = 'CakeProducts.ProductCategoryAttributes';
//        $this->_tableConfigKey = 'CakeProducts.ProductCategoryAttributes.table';
    }

    /**
     * Index method
     *
     * @return Response|null|void Renders view
     */
    public function index()
    {
        $query = $this->getTable()->find()
            ->contain(['ProductCategories']);
        $productCategoryAttributes = $this->paginate($query);

        $this->set(compact('productCategoryAttributes'));
    }

    /**
     * View method
     *
     * @param string|null $id Product Category Attribute id.
     * @return Response|null|void Renders view
     * @throws RecordNotFoundException When record not found.
     */
    public function view($id = null)
    {
        $productCategoryAttribute = $this->getTable()->get($id, contain: [
            'ProductCategories',
            'ProductCategoryAttributeOptions',
        ]);
        $this->set(compact('productCategoryAttribute'));
    }

    /**
     * Add method
     *
     * @return Response|null|void Redirects on successful add, renders view otherwise.
     */
    public function add()
    {
        $productCategoryAttributesTable = $this->getTable();
        $productCategoryAttribute = $productCategoryAttributesTable->newEmptyEntity();
        if ($this->request->is('post')) {
            $postData = $this->request->getData();
            if ($this->request->getSession()->read('Auth.User.id')) {
                $postData['created_by'] = $this->request->getSession()->read('Auth.User.id');
            }
            Log::debug(print_r('$postData', true));
            Log::debug(print_r($postData, true));
            $saveOptions = [
                'associated' => [
                    'ProductCategoryAttributeOptions'
                ],
            ];
            $productCategoryAttribute = $productCategoryAttributesTable->patchEntity($productCategoryAttribute, $postData, $saveOptions);
            if ($productCategoryAttribute->getErrors()) {
                Log::debug(print_r('$productCategoryAttribute->getErrors() next - failed to save from create new product category attribute', true));
                Log::debug(print_r($productCategoryAttribute->getErrors(), true));
            }
            if ($productCategoryAttributesTable->save($productCategoryAttribute, $saveOptions)) {
                $this->Flash->success(__('The product category attribute has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            Log::debug('failed to save new product category attribute errors next');
            Log::debug(print_r('$productCategoryAttribute->getErrors()', true));
            Log::debug(print_r($productCategoryAttribute->getErrors(), true));
            $this->Flash->error(__('The product category attribute could not be saved. Please, try again.'));
        }
        $productCategories = $productCategoryAttributesTable->ProductCategories->find('list', keyField: 'internal_id', valueField: 'name')->all();
        $this->set(compact('productCategoryAttribute', 'productCategories'));
    }

    /**
     * Edit method
     *
     * @param string|null $id Product Category Attribute id.
     * @return Response|null|void Redirects on successful edit, renders view otherwise.
     * @throws RecordNotFoundException When record not found.
     */
    public function edit($id = null)
    {
        $productCategoryAttributesTable = $this->getTable();
        $productCategoryAttribute = $productCategoryAttributesTable->get($id, contain: ['ProductCategoryAttributeOptions']);
        if ($this->request->is(['patch', 'post', 'put'])) {
            $postData = $this->request->getData();
            $saveOptions = [
                'associated' => ['ProductCategoryAttributeOptions'],
            ];
            Log::debug(print_r('$postData', true));
            Log::debug(print_r($postData, true));
//            if ($this->request->getData('attribute_type_id') != ProductCategoryAttributeTypeId::Constrained) {
//                $saveOptions['associated'] = [];
//                $postData['product_category_attribute_options'] = [];
//            }
            Log::debug(print_r('$postData', true));
            Log::debug(print_r($postData, true));
            $productCategoryAttribute = $productCategoryAttributesTable->patchEntity($productCategoryAttribute, $postData, $saveOptions);

            if ($productCategoryAttributesTable->save($productCategoryAttribute, $saveOptions)) {
                $this->Flash->success(__('The product category attribute has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            Log::debug('failed to save product category attribute on edit errors next');
            Log::debug(print_r('$productCategoryAttribute->getErrors()', true));
            Log::debug(print_r($productCategoryAttribute->getErrors(), true));
            $this->Flash->error(__('The product category attribute could not be saved. Please, try again.'));
        }
        $productCategories = $productCategoryAttributesTable->ProductCategories->find('list', limit: 200, keyField: 'internal_id', valueField: 'name')->all();
        $this->set(compact('productCategoryAttribute', 'productCategories'));
    }

    /**
     * Delete method
     *
     * @param string|null $id Product Category Attribute id.
     * @return Response|null Redirects to index.
     * @throws RecordNotFoundException When record not found.
     */
    public function delete($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);

        $productCategoryAttributesTable = $this->getTable();
        $productCategoryAttribute = $productCategoryAttributesTable->get($id);
        if ($productCategoryAttributesTable->delete($productCategoryAttribute)) {
            $this->Flash->success(__('The product category attribute has been deleted.'));
        } else {
            $this->Flash->error(__('The product category attribute could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }
}
