<?php
declare(strict_types=1);

namespace CakeProducts\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Query\SelectQuery;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use CakeProducts\Model\Entity\ProductCategoryAttributeOption;
use Closure;
use Psr\SimpleCache\CacheInterface;

/**
 * ProductCategoryAttributeOptions Model
 *
 * @property ProductCategoryAttributesTable&BelongsTo $ProductCategoryAttributes
 *
 * @method ProductCategoryAttributeOption newEmptyEntity()
 * @method ProductCategoryAttributeOption newEntity(array $data, array $options = [])
 * @method array<ProductCategoryAttributeOption> newEntities(array $data, array $options = [])
 * @method ProductCategoryAttributeOption get(mixed $primaryKey, array|string $finder = 'all', CacheInterface|string|null $cache = null, Closure|string|null $cacheKey = null, mixed ...$args)
 * @method ProductCategoryAttributeOption findOrCreate($search, ?callable $callback = null, array $options = [])
 * @method ProductCategoryAttributeOption patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method array<ProductCategoryAttributeOption> patchEntities(iterable $entities, array $data, array $options = [])
 * @method ProductCategoryAttributeOption|false save(EntityInterface $entity, array $options = [])
 * @method ProductCategoryAttributeOption saveOrFail(EntityInterface $entity, array $options = [])
 * @method iterable<ProductCategoryAttributeOption>|ResultSetInterface<ProductCategoryAttributeOption>|false saveMany(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryAttributeOption>|ResultSetInterface<ProductCategoryAttributeOption> saveManyOrFail(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryAttributeOption>|ResultSetInterface<ProductCategoryAttributeOption>|false deleteMany(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryAttributeOption>|ResultSetInterface<ProductCategoryAttributeOption> deleteManyOrFail(iterable $entities, array $options = [])
 */
class ProductCategoryAttributeOptionsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array<string, mixed> $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('product_category_attribute_options');
        $this->setDisplayField('attribute_value');
        $this->setPrimaryKey('id');

        $this->belongsTo('ProductCategoryAttributes', [
            'foreignKey' => 'product_category_attribute_id',
            'joinType' => 'INNER',
            'className' => 'CakeProducts.ProductCategoryAttributes',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->integer('product_category_attribute_id')
            ->notEmptyString('product_category_attribute_id');

        $validator
            ->scalar('attribute_value')
            ->maxLength('attribute_value', 255)
            ->requirePresence('attribute_value', 'create')
            ->notEmptyString('attribute_value');

        $validator
            ->scalar('attribute_label')
            ->maxLength('attribute_label', 255)
            ->requirePresence('attribute_label', 'create')
            ->notEmptyString('attribute_label');

        $validator
            ->boolean('enabled')
            ->notEmptyString('enabled');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
        $rules->add($rules->existsIn(['product_category_attribute_id'], 'ProductCategoryAttributes'), ['errorField' => '0']);

        return $rules;
    }
}
