<?php
declare(strict_types=1);

namespace CakeProducts\Test\TestCase\Controller;

use Cake\Log\Log;
use Cake\ORM\Table;
use Cake\TestSuite\IntegrationTestTrait;
use Cake\TestSuite\TestCase;
use CakeProducts\Controller\ExternalProductCatalogsController;
use CakeProducts\Model\Table\ExternalProductCatalogsTable;
use PHPUnit\Exception;

/**
 * CakeProducts\Controller\ExternalProductCatalogsController Test Case
 *
 * @uses \CakeProducts\Controller\ExternalProductCatalogsController
 */
class ExternalProductCatalogsControllerTest extends BaseControllerTest
{
    /**
     * Test subject table
     *
     * @var ExternalProductCatalogsTable|Table
     */
    protected $ExternalProductCatalogs;

    /**
     * Fixtures
     *
     * @var array<string>
     */
    protected array $fixtures = [
        'plugin.CakeProducts.ExternalProductCatalogs',
        'plugin.CakeProducts.ProductCatalogs',
    ];

    /**
     * setUp method
     *
     * @return void
     */
    protected function setUp(): void
    {
        parent::setUp();
//        $this->enableCsrfToken();
//        $this->enableSecurityToken();
        $this->disableErrorHandlerMiddleware();
        $this->ExternalProductCatalogs = $this->getTableLocator()->get('ExternalProductCatalogs');
    }

    /**
     * tearDown method
     *
     * @return void
     */
    protected function tearDown(): void
    {
        unset($this->ExternalProductCatalogs);

        parent::tearDown();
    }

    /**
    * Test index method
    *
    * Tests the index action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::index()
    * @throws Exception
    *
    * @return void
    */
    public function testIndexGet(): void
    {
        Log::debug('inside testIndexGet ExternalProductCatalogsControllerTest');
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'index',
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test view method
    *
    * Tests the view action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::view()
    * @throws Exception
    *
    * @return void
    */
    public function testViewGet(): void
    {
        $id = 1;
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'view',
            $id,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test add method
    *
    * Tests the add action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddGet(): void
    {
        $cntBefore = $this->ExternalProductCatalogs->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'add',
        ];
        $this->get($url);
        $this->assertResponseCode(200);

        $cntAfter = $this->ExternalProductCatalogs->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddPostSuccess(): void
    {
        $cntBefore = $this->ExternalProductCatalogs->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'add',
        ];
        $data = [
            'product_catalog_id' => 'f56f3412-ed23-490b-be6e-016208c415d2',
            'base_url' => 'http://localhost:8766',
            'api_url' => 'http://localhost:8766/api/v1/',
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('external-product-catalogs');

        $cntAfter = $this->ExternalProductCatalogs->find()->count();
        $this->assertEquals($cntBefore + 1, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddPostFailure(): void
    {
        $cntBefore = $this->ExternalProductCatalogs->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'add',
        ];
        $data = [
            'product_catalog_id' => 999999,
            'base_url' => '',
            'api_url' => 'http://localhost:8766/api/v1/',
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(200);

        $cntAfter = $this->ExternalProductCatalogs->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test edit method
    *
    * Tests the edit action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditGet(): void
    {
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'edit',
            1,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditPutSuccess(): void
    {
        $this->loginUserByRole('admin');
        $id = 1;
        $before = $this->ExternalProductCatalogs->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'edit',
            $id,
        ];
        $data = [
            'product_catalog_id' => '115153f3-2f59-4234-8ff8-e1b205761428',
            'base_url' => 'http://localhost:8766',
            'api_url' => 'http://localhost:8766/api/v1/',
            'enabled' => true,
        ];
        $this->put($url, $data);

        $this->assertResponseCode(302);
        $this->assertRedirectContains('external-product-catalogs');

        $after = $this->ExternalProductCatalogs->get($id);
        // assert saved properly below
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditPutFailure(): void
    {
        $this->loginUserByRole('admin');
        $id = 1;
        $before = $this->ExternalProductCatalogs->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'edit',
            $id,
        ];
        $data = [
            'product_catalog_id' => 9999999,
            'base_url' => '',
            'api_url' => 'http://localhost:8766/api/v1/',
            'enabled' => true,
        ];
        $this->put($url, $data);
        $this->assertResponseCode(200);
        $after = $this->ExternalProductCatalogs->get($id);

        // assert save failed below
    }

    /**
    * Test delete method
    *
    * Tests the delete action with a logged in user
    *
    * @uses \CakeProducts\Controller\ExternalProductCatalogsController::delete()
    * @throws Exception
    *
    * @return void
    */
    public function testDelete(): void
    {
        $cntBefore = $this->ExternalProductCatalogs->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ExternalProductCatalogs',
            'action' => 'delete',
            1,
        ];
        $this->delete($url);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('external-product-catalogs');

        $cntAfter = $this->ExternalProductCatalogs->find()->count();
        $this->assertEquals($cntBefore - 1, $cntAfter);
    }
}
