<?php
declare(strict_types=1);

namespace CakeProducts\Test\TestCase\Controller;

use Cake\ORM\Table;
use Cake\TestSuite\IntegrationTestTrait;
use Cake\TestSuite\TestCase;
use CakeProducts\Controller\ProductCategoriesController;
use CakeProducts\Model\Table\ProductCategoriesTable;
use PHPUnit\Exception;

/**
 * CakeProducts\Controller\ProductCategoriesController Test Case
 *
 * @uses ProductCategoriesController
 */
class ProductCategoriesControllerTest extends BaseControllerTest
{
    /**
     * Test subject table
     *
     * @var ProductCategoriesTable|Table
     */
    protected $ProductCategories;

    /**
     * Fixtures
     *
     * @var array<string>
     */
    protected array $fixtures = [
        'plugin.CakeProducts.ProductCatalogs',
        'plugin.CakeProducts.ProductCategories',
    ];

    /**
     * setUp method
     *
     * @return void
     */
    protected function setUp(): void
    {
        parent::setUp();
//        $this->enableCsrfToken();
//        $this->enableSecurityToken();
        $this->disableErrorHandlerMiddleware();
        $this->ProductCategories = $this->getTableLocator()->get('ProductCategories');
    }

    /**
     * tearDown method
     *
     * @return void
     */
    protected function tearDown(): void
    {
        unset($this->ProductCategories);

        parent::tearDown();
    }

    /**
    * Test index method
    *
    * Tests the index action with a logged in user
    *
    * @return void
     * @throws Exception
     *
    * @uses ProductCategoriesController::index
    */
    public function testIndexGet(): void
    {
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'index',
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test view method
    *
    * Tests the view action with a logged in user
    *
    * @return void
     * @throws Exception
     *
    * @uses ProductCategoriesController::view
    */
    public function testViewGet(): void
    {
        $id = 1;
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'view',
            $id,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test add method
    *
    * Tests the add action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoriesController::add
    */
    public function testAddGet(): void
    {
        $cntBefore = $this->ProductCategories->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'add',
        ];
        $this->get($url);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCategories->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoriesController::add
    */
    public function testAddPostSuccess(): void
    {
        $cntBefore = $this->ProductCategories->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'add',
        ];
        $data = [
            'name' => 'Electrical Plugs',
            'product_catalog_id' => '115153f3-2f59-4234-8ff8-e1b205761428',
            'category_description' => 'electrical',
            'parent_id' => 3,
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-categories');

        $cntAfter = $this->ProductCategories->find()->count();
        $this->assertEquals($cntBefore + 1, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoriesController::add
    */
    public function testAddPostFailure(): void
    {
        $cntBefore = $this->ProductCategories->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'add',
        ];
        $data = [
            'name' => '',
            'product_catalog_id' => '',
            'category_description' => 'electrical',
            'parent_id' => '',
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCategories->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test edit method
    *
    * Tests the edit action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoriesController::edit
    */
    public function testEditGet(): void
    {
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'edit',
            1,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoriesController::edit
    */
    public function testEditPutSuccess(): void
    {
        $this->loginUserByRole('admin');
        $id = 1;
        $before = $this->ProductCategories->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'edit',
            $id,
        ];
        $data = [
            // test new data here
            'name' => 'Electrical v2',
            'product_catalog_id' => '115153f3-2f59-4234-8ff8-e1b205761428',
            'category_description' => 'electrical v2',
            'parent_id' => '',
            'enabled' => true,
        ];
        $this->put($url, $data);

        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-categories');

        $after = $this->ProductCategories->get($id);
        $this->assertEquals($data['name'], $after->name);
        $this->assertEquals($data['product_catalog_id'], $after->product_catalog_id);
        $this->assertEquals($data['category_description'], $after->category_description);
        $this->assertNull($after->parent_id);
        // assert saved properly below
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoriesController::edit
    */
    public function testEditPutFailure(): void
    {
        $this->loginUserByRole('admin');
        $id = 1;
        $before = $this->ProductCategories->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'edit',
            $id,
        ];
        $data = [
            'name' => '',
            'product_catalog_id' => '',
            'category_description' => 'electrical',
            'parent_id' => '',
            'enabled' => true,
        ];
        $this->put($url, $data);
        $this->assertResponseCode(200);
        $after = $this->ProductCategories->get($id);

        // assert save failed below
    }

    /**
    * Test delete method
    *
    * Tests the delete action with a logged in user
    *
    * @return void
    *@throws Exception
     *
     * @uses ProductCategoriesController::delete
    */
    public function testDelete(): void
    {
        $cntBefore = $this->ProductCategories->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategories',
            'action' => 'delete',
            1,
        ];
        $this->delete($url);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-categories');

        $cntAfter = $this->ProductCategories->find()->count();
        $this->assertEquals($cntBefore - 1, $cntAfter);
    }
}
