<?php
declare(strict_types=1);

namespace CakeProducts\Test\TestCase\Controller;

use Cake\ORM\Table;
use CakeProducts\Controller\ProductCategoryAttributeOptionsController;
use CakeProducts\Model\Table\ProductCategoryAttributeOptionsTable;
use PHPUnit\Exception;

/**
 * CakeProducts\Controller\ProductCategoryAttributeOptionsController Test Case
 *
 * @uses \CakeProducts\Controller\ProductCategoryAttributeOptionsController
 */
class ProductCategoryAttributeOptionsControllerTest extends BaseControllerTest
{
    /**
     * Test subject
     *
     * @var ProductCategoryAttributeOptionsTable|Table
     */
    protected $ProductCategoryAttributeOptions;

    /**
     * Fixtures
     *
     * @var array<string>
     */
    protected array $fixtures = [
        'plugin.CakeProducts.ProductCategoryAttributeOptions',
        'plugin.CakeProducts.ProductCategoryAttributes',
    ];

    /**
     * setUp method
     *
     * @return void
     */
    protected function setUp(): void
    {
        parent::setUp();
//        $this->enableCsrfToken();
//        $this->enableSecurityToken();
        $config = $this->getTableLocator()->exists('ProductCategoryAttributeOptions') ? [] : ['className' => ProductCategoryAttributeOptionsTable::class];
        $this->ProductCategoryAttributeOptions = $this->getTableLocator()->get('ProductCategoryAttributeOptions', $config);
    }

    /**
     * tearDown method
     *
     * @return void
     */
    protected function tearDown(): void
    {
        unset($this->ProductCategoryAttributeOptions);

        parent::tearDown();
    }

    /**
     * Test add method
     *
     * Tests the add action with a logged in user
     *
     * @return void
     * @throws Exception
     *
     * @uses CustomersContactsController::add
     */
    public function testAddGet(): void
    {
        $cntBefore = $this->ProductCategoryAttributeOptions->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributeOptions',
            'action' => 'add',
        ];
        $this->get($url);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCategoryAttributeOptions->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
     * Test add method
     *
     * Tests a POST request to the add action with a logged in user
     *
     * @return void
     * @throws Exception
     *
     * @uses CustomersContactsController::add
     */
    public function testAddPostHasNoEffect(): void
    {
        $cntBefore = $this->ProductCategoryAttributeOptions->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributeOptions',
            'action' => 'add',
        ];
        $data = [];
        $this->post($url, $data);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCategoryAttributeOptions->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
     * Test delete method
     *
     * Tests the delete action with a logged in user
     *
     * @return void
     *@throws Exception
     *
     * @uses CustomersContactsController::delete
     */
    public function testDelete(): void
    {
        $cntBefore = $this->ProductCategoryAttributeOptions->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributeOptions',
            'action' => 'delete',
            'e06f1723-2456-483a-b3c4-004603e032a8',
        ];
        $this->delete($url);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-category-attributes');

        $cntAfter = $this->ProductCategoryAttributeOptions->find()->count();
        $this->assertEquals($cntBefore - 1, $cntAfter);
    }
}
