<?php
declare(strict_types=1);

namespace CakeProducts\Test\TestCase\Controller;

use Cake\ORM\Table;
use Cake\TestSuite\IntegrationTestTrait;
use Cake\TestSuite\TestCase;
use CakeProducts\Controller\ProductCategoryAttributesController;
use CakeProducts\Model\Table\ProductCategoryAttributesTable;
use PHPUnit\Exception;

/**
 * CakeProducts\Controller\ProductCategoryAttributesController Test Case
 *
 * @uses ProductCategoryAttributesController
 */
class ProductCategoryAttributesControllerTest extends BaseControllerTest
{
    /**
     * Test subject
     *
     * @var ProductCategoryAttributesTable|Table
     */
    protected $ProductCategoryAttributes;

    /**
     * Fixtures
     *
     * @var array<string>
     */
    protected array $fixtures = [
        'plugin.CakeProducts.ProductCategoryAttributes',
        'plugin.CakeProducts.ProductCategories',
    ];

    /**
     * setUp method
     *
     * @return void
     */
    protected function setUp(): void
    {
        parent::setUp();
//        $this->enableCsrfToken();
//        $this->enableSecurityToken();
        $config = $this->getTableLocator()->exists('ProductCategoryAttributes') ? [] : ['className' => ProductCategoryAttributesTable::class];
        $this->ProductCategoryAttributes = $this->getTableLocator()->get('ProductCategoryAttributes', $config);
    }

    /**
     * tearDown method
     *
     * @return void
     */
    protected function tearDown(): void
    {
        unset($this->ProductCategoryAttributes);

        parent::tearDown();
    }

    /**
    * Test index method
    *
    * Tests the index action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::index
    */
    public function testIndexGet(): void
    {
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'index',
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test view method
    *
    * Tests the view action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::view
    */
    public function testViewGet(): void
    {
        $id = '37078cf0-0130-4b93-bb7e-abe7d665ed2c';
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'view',
            $id,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test add method
    *
    * Tests the add action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::add
    */
    public function testAddGet(): void
    {
        $cntBefore = $this->ProductCategoryAttributes->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'add',
        ];
        $this->get($url);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCategoryAttributes->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::add
    */
    public function testAddPostSuccess(): void
    {
        $cntBefore = $this->ProductCategoryAttributes->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'add',
        ];
        $data = [
            'name' => 'Size',
            'product_category_id' => 'db4b4273-eddc-46d4-93c8-45cf7c6e058e',
            'attribute_type_id' => 2,
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-category-attributes');

        $cntAfter = $this->ProductCategoryAttributes->find()->count();
        $this->assertEquals($cntBefore + 1, $cntAfter);
    }

    /**
     * Test add method
     *
     * Tests a POST request to the add action with a logged in user
     *
     * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::add
     */
    public function testAddPostSuccessConstrainedWithOptions(): void
    {
        $cntBefore = $this->ProductCategoryAttributes->find()->count();
        $cntOptionsBefore = $this->ProductCategoryAttributes->ProductCategoryAttributeOptions->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'add',
        ];
        $data = [
            'name' => 'Size',
            'product_category_id' => 'db4b4273-eddc-46d4-93c8-45cf7c6e058e',
            'attribute_type_id' => 1,
            'enabled' => true,
            'product_category_attribute_options' => [
                [
                    'attribute_value' => 'XL',
                    'attribute_label' => 'XL',
                    'enabled' => true,
                ],
                [
                    'attribute_value' => 'L',
                    'attribute_label' => 'L',
                    'enabled' => true,
                ]
            ],
        ];
        $this->post($url, $data);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-category-attributes');

        $cntAfter = $this->ProductCategoryAttributes->find()->count();
        $cntOptionsAfter = $this->ProductCategoryAttributes->ProductCategoryAttributeOptions->find()->count();

        $this->assertEquals($cntBefore + 1, $cntAfter);
        $this->assertEquals($cntOptionsBefore + 2, $cntOptionsAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::add
    */
    public function testAddPostFailure(): void
    {
        $cntBefore = $this->ProductCategoryAttributes->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'add',
        ];
        $data = [
            'name' => '',
            'product_category_id' => 1,
            'attribute_type_id' => 1,
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCategoryAttributes->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test edit method
    *
    * Tests the edit action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::edit
    */
    public function testEditGet(): void
    {
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'edit',
            '37078cf0-0130-4b93-bb7e-abe7d665ed2c',
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::edit
    */
    public function testEditPutSuccess(): void
    {
        $this->loginUserByRole('admin');
        $id = '37078cf0-0130-4b93-bb7e-abe7d665ed2c';
        $before = $this->ProductCategoryAttributes->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'edit',
            $id,
        ];
        $data = [
            // test new data here
            'name' => 'Color',
            'product_category_id' => 'db4b4273-eddc-46d4-93c8-45cf7c6e058e',
            'attribute_type_id' => 1,
            'enabled' => true,
        ];
        $this->put($url, $data);

        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-category-attributes');

        $after = $this->ProductCategoryAttributes->get($id);
        // assert saved properly below
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @return void
     * @throws Exception
     *
     * @uses ProductCategoryAttributesController::edit
    */
    public function testEditPutFailure(): void
    {
        $this->loginUserByRole('admin');
        $id = '37078cf0-0130-4b93-bb7e-abe7d665ed2c';
        $before = $this->ProductCategoryAttributes->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'edit',
            $id,
        ];
        $data = [
            'name' => '',
            'product_category_id' => 1,
            'attribute_type_id' => 1,
            'enabled' => true,
        ];
        $this->put($url, $data);
        $this->assertResponseCode(200);
        $after = $this->ProductCategoryAttributes->get($id);

        // assert save failed below
    }

    /**
    * Test delete method
    *
    * Tests the delete action with a logged in user
    *
    * @return void
    *@throws Exception
     *
     * @uses ProductCategoryAttributesController::delete
    */
    public function testDelete(): void
    {
        $cntBefore = $this->ProductCategoryAttributes->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCategoryAttributes',
            'action' => 'delete',
            '37078cf0-0130-4b93-bb7e-abe7d665ed2c',
        ];
        $this->delete($url);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-category-attributes');

        $cntAfter = $this->ProductCategoryAttributes->find()->count();
        $this->assertEquals($cntBefore - 1, $cntAfter);
    }
}
